@@ -30,12 +30,19 @@ PRIMER3_HOME = os.environ.get('PRIMER3HOME')
 
 _primer3.getThermoParams(pjoin(PRIMER3_HOME, 'src', 'primer3_config/'))
 
+
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Low level bindings ~~~~~~~~~~~~~~~~~~~~~~~~~~~ #
 
-# Named tuple returned by all low-level bindings (if a structure is not 
+# Named tuple returned by all low-level bindings (if a structure is not
 # present, the functions simply return `None`)
-THERMORESULT = namedtuple('thermoresult', ['tm', 'ds', 'dh', 'dg',
-                                           'align_end_1', 'align_end_2'])
+THERMORESULT = namedtuple('thermoresult', [
+    'tm',               # Melting temperature (deg. Celsius)
+    'ds',               # Entropy (cal/(K*mol))
+    'dh',               # Enthalpy (kcal/mol)
+    'dg',               # Gibbs free energy
+    'align_end_1',      # Last alignment position in the 1st sequence.
+    'align_end_2']      # Last alignment position in the 2nd sequence.
+)
 
 
 def _calcThermo(seq1, seq2, calc_type=0, mv_conc=50, dv_conc=0, dntp_conc=0.8,
@@ -53,8 +60,7 @@ def _calcThermo(seq1, seq2, calc_type=0, mv_conc=50, dv_conc=0, dntp_conc=0.8,
 
 def calcHairpin(seq, mv_conc=50.0, dv_conc=0.0, dntp_conc=0.8, dna_conc=50.0,
                 temp_c=37, max_loop=30):
-    ''' Return a `thermoresult` named tuple hairpin structure characteristics
-    if present, otherwise, return None.
+    ''' Calculate the hairpin formation thermodynamics of a DNA sequence.
 
     Args:
         seq (str)               : DNA sequence to analyze for hairpin formation
@@ -67,6 +73,10 @@ def calcHairpin(seq, mv_conc=50.0, dv_conc=0.0, dntp_conc=0.8, dna_conc=50.0,
         temp_c (int)            : Simulation temperature for dG (Celsius)
         max_loop(int)           : Maximum size of loops in the structure
 
+    Returns:
+        A `thermoresult` named tuple of thermodynamic characteristics of the
+        hairpin formation. If no hairpin is formed, returns `None`.
+
     '''
 
     return _calcThermo(seq, seq, 4, mv_conc, dv_conc, dntp_conc, dna_conc,
@@ -75,11 +85,10 @@ def calcHairpin(seq, mv_conc=50.0, dv_conc=0.0, dntp_conc=0.8, dna_conc=50.0,
 
 def calcHomodimer(seq, mv_conc=50, dv_conc=0, dntp_conc=0.8, dna_conc=50,
                   temp_c=37, max_loop=30):
-    ''' Return a `thermoresult` named tuple of homodimer characteristics if
-    present, otherwise, return None.
+    ''' Calculate the homodimerization thermodynamics of a DNA sequence.
 
     Args:
-        seq (str)               : DNA sequence to analyze for homodimer 
+        seq (str)               : DNA sequence to analyze for homodimer
                                   formation calculations
 
     Kwargs:
@@ -88,7 +97,11 @@ def calcHomodimer(seq, mv_conc=50, dv_conc=0, dntp_conc=0.8, dna_conc=50,
         dntp_conc (float/int)   : dNTP concentration (mM)
         dna_conc (float/int)    : DNA concentration (nM)
         temp_c (int)            : Simulation temperature for dG (Celsius)
-        max_loop(int)           : Maximum size of loops in the structure
+        max_loop (int)          : Maximum size of loops in the structure
+
+    Returns:
+        A `thermoresult` named tuple of thermodynamic characteristics of the
+        homodimer interaction. If no interaction is present, returns `None`.
 
     '''
     return _calcThermo(seq, seq, 1, mv_conc, dv_conc, dntp_conc, dna_conc,
@@ -97,13 +110,12 @@ def calcHomodimer(seq, mv_conc=50, dv_conc=0, dntp_conc=0.8, dna_conc=50,
 
 def calcHeterodimer(seq1, seq2, mv_conc=50, dv_conc=0, dntp_conc=0.8,
                     dna_conc=50, temp_c=37, max_loop=30):
-    ''' Return a `thermoresult` named tuple of heterodimer characteristics if
-    present, otherwise, return None.
+    ''' Calculate the heterodimerization thermodynamics of two DNA sequences.
 
     Args:
-        seq1 (str)              : First DNA sequence to analyze for heterodimer 
+        seq1 (str)              : First DNA sequence to analyze for heterodimer
                                   formation
-        seq2 (str)              : Second DNA sequence to analyze for  
+        seq2 (str)              : Second DNA sequence to analyze for
                                   heterodimer formation
 
     Kwargs:
@@ -114,6 +126,10 @@ def calcHeterodimer(seq1, seq2, mv_conc=50, dv_conc=0, dntp_conc=0.8,
         temp_c (int)            : Simulation temperature for dG (Celsius)
         max_loop(int)           : Maximum size of loops in the structure
 
+    Returns:
+        A `thermoresult` named tuple of thermodynamic characteristics of the
+        heterodimer interaction. If no interaction is present, returns `None`.
+
     '''
     return _calcThermo(seq1, seq2, 1, mv_conc, dv_conc, dntp_conc, dna_conc,
                        temp_c, max_loop)
@@ -131,8 +147,29 @@ _salt_correction_methods = {
 }
 
 def calcTm(seq, mv_conc=50, dv_conc=0, dntp_conc=0.8, dna_conc=50,
-                  max_nn_length=60, tm_method='santalucia',
-                  salt_corrections_method='santalucia'):
+           max_nn_length=60, tm_method='santalucia',
+           salt_corrections_method='santalucia'):
+    ''' Calculate the melting temperature of a DNA sequence.
+
+    Args:
+        seq (str)               : DNA sequence
+
+    Kwargs:
+        mv_conc (float/int)     : Monovalent cation concentration (mM)
+        dv_conc (float/int)     : Divalent cation concentration (mM)
+        dntp_conc (float/int)   : dNTP concentration (mM)
+        dna_conc (float/int)    : DNA concentration (nM)
+        max_nn_length (int)     : Maximum length for nearest-neighbor calcs
+        tm_method (str)         : Tm calculation method (breslauer or
+                                  santalucia)
+        salt_corrections_method
+                          (str) : Salt correction method (schildkraut,
+                                  owczarzy, santalucia)
+
+    Returns:
+        The melting temperature in degrees Celsius (float).
+
+    '''
     tm_meth = _tm_methods.get(tm_method)
     if not tm_meth:
         raise ValueError('{} is not a valid tm calculation method'.format(
@@ -148,6 +185,41 @@ def calcTm(seq, mv_conc=50, dv_conc=0, dntp_conc=0.8, dna_conc=50,
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Design bindings ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ #
 
 
+def designPrimers(seq_args, global_args=None, misprime_lib=None,
+                  mishyb_lib=None):
+    ''' Run the Primer3 design process.
+
+    If the global args have been previously set (either by a pervious
+    `designPrimers` call or by a `setGlobals` call), `designPrimers` may be
+    called with seqArgs alone (as a means of optimization).
+
+    Args:
+        seq_args (dict)     :   Primer3 sequence/design args as per Primer3 docs
+
+    Kwargs:
+        global_args (dict)  :   Primer3 global args as per Primer3 docs
+        misprime_lib (dict) :   `Sequence name: sequence` dictionary for
+                                mispriming checks.
+        mishyb_lib (dict)   :   `Sequence name: sequence` dictionary for
+                                mishybridization checks.
+
+    Returns:
+        A dictionary of Primer3 results (should be identical to the expected
+        BoulderIO output from primer3_main)
+
+    '''
+    if global_args:
+        _primer3.setGlobals(global_args, misprime_lib, mishyb_lib)
+    _primer3.setGlobals(global_args, misprime_lib, mishyb_lib)
+    _primer3.setSeqArgs(seq_args)
+    return _primer3.runDesign()
+
+
+'''
+The following functions are the modular subunits of `designPrimers` and may
+be used in cases where performance or customiziation are of high priority.
+'''
+
 def setP3Globals(global_args, misprime_lib=None, mishyb_lib=None):
     ''' Set the Primer3 global args and misprime/mishyb libraries.
 
@@ -177,35 +249,12 @@ def setP3SeqArgs(seq_args):
 def runP3Design():
     ''' Start the Primer3 design process, return a dict of the Primer3 output.
 
-    The global parameters and seq args must have been previously set prior to 
+    The global parameters and seq args must have been previously set prior to
     this call (raises IOError).
 
+    Returns:
+        A dictionary of Primer3 results (should be identical to the expected
+        BoulderIO output from primer3_main)
+
     '''
     _primer3.runDesign()
-
-
-def designPrimers(seq_args, global_args=None, misprime_lib=None, 
-                  mishyb_lib=None):
-    ''' Run the Primer3 design process, return a dict of the Primer3 output.
-
-    If the global args have been previously set (either by a pervious
-    `designPrimers` call or by a `setGlobals` call), `designPrimers` may be 
-    called with seqArgs alone (as a means of optimization).
-
-    Args:
-        seq_args (dict)     :   Primer3 sequence/design args as per Primer3 docs
-
-    Kwargs:
-        global_args (dict)  :   Primer3 global args as per Primer3 docs
-        misprime_lib (dict) :   `Sequence name: sequence` dictionary for
-                                mispriming checks.
-        mishyb_lib (dict)   :   `Sequence name: sequence` dictionary for
-                                mishybridization checks.
-
-    '''
-    if global_args:
-        _primer3.setGlobals(global_args, misprime_lib, mishyb_lib)
-    _primer3.setGlobals(global_args, misprime_lib, mishyb_lib)
-    _primer3.setSeqArgs(seq_args)
-    return _primer3.runDesign()
-
